import { ServiceIdentifier } from '../../instantiation/common/instantiation.js';
import { URI } from '../../../base/common/uri.js';

declare const IClipboardService: ServiceIdentifier<IClipboardService>;
interface IClipboardService {
    readonly _serviceBrand: undefined;
    /**
    * Writes text to the system clipboard.
    */
    writeText(text: string, type?: string): Promise<void>;
    /**
    * Reads the content of the clipboard in plain text
    */
    readText(type?: string): Promise<string>;
    /**
    * Reads text from the system find pasteboard.
    */
    readFindText(): Promise<string>;
    /**
    * Writes text to the system find pasteboard.
    */
    writeFindText(text: string): Promise<void>;
    /**
    * Writes resources to the system clipboard.
    */
    writeResources(resources: URI[]): Promise<void>;
    /**
    * Reads resources from the system clipboard.
    */
    readResources(): Promise<URI[]>;
    /**
    * Find out if resources are copied to the clipboard.
    */
    hasResources(): Promise<boolean>;
}

export { IClipboardService };
