import { Color, RGBA } from '../../../../base/common/color.js';
import { localizeWithPath } from '../../../../nls.js';
import { registerColor, transparent } from '../../../../platform/theme/common/colorUtils.js';
import { focusBorder } from '../../../../platform/theme/common/colors/baseColors.js';
import '../../../../platform/theme/common/colors/chartsColors.js';
import { editorWidgetBorder } from '../../../../platform/theme/common/colors/editorColors.js';
import { selectBackground, selectForeground, selectBorder, checkboxBackground, checkboxForeground, checkboxBorder, inputBackground, inputForeground, inputBorder } from '../../../../platform/theme/common/colors/inputColors.js';
import { listHoverBackground } from '../../../../platform/theme/common/colors/listColors.js';
import '../../../../platform/theme/common/colors/menuColors.js';
import '../../../../platform/theme/common/colors/minimapColors.js';
import '../../../../platform/theme/common/colors/miscColors.js';
import '../../../../platform/theme/common/colors/quickpickColors.js';
import '../../../../platform/theme/common/colors/searchColors.js';
import { PANEL_BORDER } from '../../../common/theme.js';
const _moduleId = "vs/workbench/contrib/preferences/common/settingsEditorColorRegistry";
const settingsHeaderForeground = registerColor('settings.headerForeground', { light: '#444444', dark: '#e7e7e7', hcDark: '#ffffff', hcLight: '#292929' }, ( localizeWithPath(_moduleId, 0, "The foreground color for a section header or active title.")));
const settingsHeaderHoverForeground = registerColor('settings.settingsHeaderHoverForeground', { light: ( (transparent(settingsHeaderForeground, 0.7))), dark: ( (transparent(settingsHeaderForeground, 0.7))), hcDark: ( (transparent(settingsHeaderForeground, 0.7))), hcLight: ( (transparent(settingsHeaderForeground, 0.7))) }, ( localizeWithPath(
    _moduleId,
    1,
    "The foreground color for a section header or hovered title."
)));
registerColor('settings.modifiedItemIndicator', {
    light: ( (new Color( (new RGBA(102, 175, 224))))),
    dark: ( (new Color( (new RGBA(12, 125, 157))))),
    hcDark: ( (new Color( (new RGBA(0, 73, 122))))),
    hcLight: ( (new Color( (new RGBA(102, 175, 224))))),
}, ( localizeWithPath(_moduleId, 2, "The color of the modified setting indicator.")));
const settingsHeaderBorder = registerColor('settings.headerBorder', { dark: PANEL_BORDER, light: PANEL_BORDER, hcDark: PANEL_BORDER, hcLight: PANEL_BORDER }, ( localizeWithPath(_moduleId, 3, "The color of the header container border.")));
const settingsSashBorder = registerColor('settings.sashBorder', { dark: PANEL_BORDER, light: PANEL_BORDER, hcDark: PANEL_BORDER, hcLight: PANEL_BORDER }, ( localizeWithPath(_moduleId, 4, "The color of the Settings editor splitview sash border.")));
const settingsSelectBackground = registerColor(`settings.dropdownBackground`, { dark: selectBackground, light: selectBackground, hcDark: selectBackground, hcLight: selectBackground }, ( localizeWithPath(_moduleId, 5, "Settings editor dropdown background.")));
const settingsSelectForeground = registerColor('settings.dropdownForeground', { dark: selectForeground, light: selectForeground, hcDark: selectForeground, hcLight: selectForeground }, ( localizeWithPath(_moduleId, 6, "Settings editor dropdown foreground.")));
const settingsSelectBorder = registerColor('settings.dropdownBorder', { dark: selectBorder, light: selectBorder, hcDark: selectBorder, hcLight: selectBorder }, ( localizeWithPath(_moduleId, 7, "Settings editor dropdown border.")));
const settingsSelectListBorder = registerColor('settings.dropdownListBorder', { dark: editorWidgetBorder, light: editorWidgetBorder, hcDark: editorWidgetBorder, hcLight: editorWidgetBorder }, ( localizeWithPath(
    _moduleId,
    8,
    "Settings editor dropdown list border. This surrounds the options and separates the options from the description."
)));
registerColor('settings.checkboxBackground', { dark: checkboxBackground, light: checkboxBackground, hcDark: checkboxBackground, hcLight: checkboxBackground }, ( localizeWithPath(_moduleId, 9, "Settings editor checkbox background.")));
registerColor('settings.checkboxForeground', { dark: checkboxForeground, light: checkboxForeground, hcDark: checkboxForeground, hcLight: checkboxForeground }, ( localizeWithPath(_moduleId, 10, "Settings editor checkbox foreground.")));
registerColor('settings.checkboxBorder', { dark: checkboxBorder, light: checkboxBorder, hcDark: checkboxBorder, hcLight: checkboxBorder }, ( localizeWithPath(_moduleId, 11, "Settings editor checkbox border.")));
const settingsTextInputBackground = registerColor('settings.textInputBackground', { dark: inputBackground, light: inputBackground, hcDark: inputBackground, hcLight: inputBackground }, ( localizeWithPath(_moduleId, 12, "Settings editor text input box background.")));
const settingsTextInputForeground = registerColor('settings.textInputForeground', { dark: inputForeground, light: inputForeground, hcDark: inputForeground, hcLight: inputForeground }, ( localizeWithPath(_moduleId, 13, "Settings editor text input box foreground.")));
const settingsTextInputBorder = registerColor('settings.textInputBorder', { dark: inputBorder, light: inputBorder, hcDark: inputBorder, hcLight: inputBorder }, ( localizeWithPath(_moduleId, 14, "Settings editor text input box border.")));
const settingsNumberInputBackground = registerColor('settings.numberInputBackground', { dark: inputBackground, light: inputBackground, hcDark: inputBackground, hcLight: inputBackground }, ( localizeWithPath(_moduleId, 15, "Settings editor number input box background.")));
const settingsNumberInputForeground = registerColor('settings.numberInputForeground', { dark: inputForeground, light: inputForeground, hcDark: inputForeground, hcLight: inputForeground }, ( localizeWithPath(_moduleId, 16, "Settings editor number input box foreground.")));
const settingsNumberInputBorder = registerColor('settings.numberInputBorder', { dark: inputBorder, light: inputBorder, hcDark: inputBorder, hcLight: inputBorder }, ( localizeWithPath(_moduleId, 17, "Settings editor number input box border.")));
registerColor('settings.focusedRowBackground', {
    dark: ( (transparent(listHoverBackground, .6))),
    light: ( (transparent(listHoverBackground, .6))),
    hcDark: null,
    hcLight: null,
}, ( localizeWithPath(_moduleId, 18, "The background color of a settings row when focused.")));
registerColor('settings.rowHoverBackground', {
    dark: ( (transparent(listHoverBackground, .3))),
    light: ( (transparent(listHoverBackground, .3))),
    hcDark: null,
    hcLight: null
}, ( localizeWithPath(_moduleId, 19, "The background color of a settings row when hovered.")));
registerColor('settings.focusedRowBorder', {
    dark: focusBorder,
    light: focusBorder,
    hcDark: focusBorder,
    hcLight: focusBorder
}, ( localizeWithPath(
    _moduleId,
    20,
    "The color of the row's top and bottom border when the row is focused."
)));
export { settingsHeaderBorder, settingsHeaderForeground, settingsHeaderHoverForeground, settingsNumberInputBackground, settingsNumberInputBorder, settingsNumberInputForeground, settingsSashBorder, settingsSelectBackground, settingsSelectBorder, settingsSelectForeground, settingsSelectListBorder, settingsTextInputBackground, settingsTextInputBorder, settingsTextInputForeground };
